#!/bin/sh

DISTRO_DIR=$1
PORT="${2:-13000}"
TMPDIR="${TMPDIR:-/tmp}"

if [ ! -d "$DISTRO_DIR" ]; then
    echo "Must specify location of Corda distribution (directory does not exist: $DISTRO_DIR)"
    exit 1
fi

if [ ! -f "$DISTRO_DIR/corda.jar" ]; then
    echo "Distribution corda.jar not found"
    exit 1
fi

if [ ! -f "$DISTRO_DIR/corda-rpcProxy.jar" ]; then
    echo "Distribution corda-rpcProxy.jar not found"
    exit 1
fi

# unzip corda jars into proxy directory (if not already there)
if [ ! -d "$DISTRO_DIR/proxy" ]; then
    mkdir -p ${DISTRO_DIR}/proxy
    unzip ${DISTRO_DIR}/corda.jar -d ${DISTRO_DIR}/proxy
fi

# launch proxy
echo "Launching RPC proxy ..."
echo "java -cp $DISTRO_DIR/corda-rpcProxy.jar:\
\n\t$(ls ${DISTRO_DIR}/proxy/*.jar | tr '\n' ':'):\
\n\t$(ls ${DISTRO_DIR}/apps/*.jar | tr '\n' ':')
\
\n\tnet.corda.behave.service.proxy.RPCProxyServerKt ${PORT}
"

/usr/bin/java -cp ${DISTRO_DIR}/corda-rpcProxy.jar:\
$(ls ${DISTRO_DIR}/proxy/*.jar | tr '\n' ':'):\
$(ls ${DISTRO_DIR}/apps/*.jar | tr '\n' ':') \
net.corda.behave.service.proxy.RPCProxyServerKt ${PORT} &> rpcproxy-${PORT}.log &

echo $! > ${TMPDIR}/rpcProxy-pid-${PORT}
echo "RPCProxyServer PID: $(cat ${TMPDIR}/rpcProxy-pid-${PORT})"
